﻿using Caliburn.Micro;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel.Composition;
using System.Linq;
using System.Windows;
using VisualComponents.UX.Shared;

namespace Plugin.GalleryMenuToolCommand
{
    [Export(typeof(IActionItem))]
    public class GalleryMenuToolCommandActionItem : ActionItem, 
                                                    IGalleryMenuToolCommand, 
                                                    IPartImportsSatisfiedNotification
    {
        public GalleryMenuToolCommandActionItem()
            : base("CustomGallery", "Create Component", @"Gallery/PrimitiveGeometry/rCube", null, MenuTools.GalleryMenuTool, true)
        {
            AttachTo = "Execute";
            ToolTipVisibility = Visibility.Visible;

            ToolTipHeader = "Tool Tip Header";
            ToolTipContent = "Tool Tip Content";
        
            ToolTipHeaderSeparatorVisibility = Visibility.Collapsed;
            ToolTipFooterSeparatorVisibility = Visibility.Collapsed;
            ToolSize = ToolSizingMode.ImageAndTextLarge;

            MinDropDownColumns = 8;
            MaxDropDownColumns = 8;
            MinPreviewColumns = 4;
            MaxPreviewColumns = 4;
            PreferredDropDownColumns = 10;
            ShouldDisplayGalleryPreview = false;
            locateGalleryMenuToolInConfig();
        }

        void IPartImportsSatisfiedNotification.OnImportsSatisfied()
        {
            EvaluateCanExecute();
        }

        private void locateGalleryMenuToolInConfig()
        {
            List<UXSiteConfigElement> GalleryGroups = new List<UXSiteConfigElement>();
            var uxConfig = IoC.Get<IUXConfiguration>();
            var groupsFound = false;

            var tabSites = uxConfig.Sites.ToList();
            if (tabSites != null)
            {
                foreach (var tabSite in tabSites)
                {
                    foreach (var site in tabSite.UXSites.ToList())
                    {
                        //find the control
                        var actionItemControl = site.UXSiteEntries.ToList().Where(item => item.ControlType == "GalleryMenuTool" & item.ItemId == "CustomGallery").FirstOrDefault();

                        if (actionItemControl != null)
                        {
                            var subItemKey = actionItemControl.SubItemsKey;
                            //find the control SubItem site
                            var subItem = site.UXSites.ToList().Where(item => item.Type == "GalleryMenuToolSubItem" & item.ItemId == subItemKey).FirstOrDefault();

                            if (subItem != null)
                            {
                                //find the subItem group
                                var groupItems = subItem.UXSites.ToList();

                                if (groupItems != null)
                                {
                                    foreach (var group in groupItems)
                                    {
                                        GalleryGroups.Add(group);
                                    }
                                    groupsFound = true;
                                }
                            }
                        }
                        if (groupsFound)
                        {
                            break;
                        }
                    }
                    if (groupsFound)
                    {
                        break;
                    }
                }
            }
            generateGalleryItem(GalleryGroups);
        }

        private void generateGalleryItem(List<UXSiteConfigElement> GalleryGroups)
        {
            var Items = new List<GalleryItemSetup>();

            GroupItems = new ObservableCollection<GalleryGroup>();

            foreach (var group in GalleryGroups)
            {
                var galleryGroup = new GalleryGroup()
                {
                    Key = group.ItemId,
                    Title = Localization.GetText(group.TitleKey),
                    ItemsKeys = new List<string>()
                };

                var galleryEntries = group.UXSiteEntries.ToList();

                foreach (var galleryEntry in galleryEntries)
                {
                    var entryConfigElement = (galleryEntry as UXSiteEntryConfigElement);
                    galleryGroup.ItemsKeys.Add(entryConfigElement.ItemId);

                    var newItem = new GalleryItemSetup()
                    {
                        Key = entryConfigElement.ItemId,
                        Text = Localization.GetText(String.IsNullOrEmpty(entryConfigElement.CaptionKey) ? entryConfigElement.ItemId : entryConfigElement.CaptionKey),
                        Priority = entryConfigElement.Priority,
                        Tag = entryConfigElement.IconKey
                    };
                    SvgImage.GetIconPathDictionary(newItem.Tag);
                    Items.Add(newItem);

                }
                GroupItems.Add(galleryGroup);
            }

            GalleryItems = new ObservableCollection<GalleryItemEx>();

            foreach (var galleryItem in Items.OrderBy(x => x.Priority))
            {
                GalleryItems.Add(new GalleryItemEx { Key = galleryItem.Key, Text = galleryItem.Text, Image = galleryItem.Image, Tag = galleryItem.Tag });
            }
        }

        private ObservableCollection<GalleryGroup> _groupItems;
        public ObservableCollection<GalleryGroup> GroupItems
        {
            get
            {
                return _groupItems;
            }
            set
            {
                _groupItems = value;
                NotifyOfPropertyChange(() => GroupItems);
            }
        }

        private ObservableCollection<GalleryItemEx> _gallaryItems;
        public ObservableCollection<GalleryItemEx> GalleryItems
        {
            get
            {
                return _gallaryItems;
            }
            set
            {
                _gallaryItems = value;
                NotifyOfPropertyChange(() => GalleryItems);
            }          
        }

        private int _minDropDownColumns;
        public int MinDropDownColumns
        {
            get
            {
                return _minDropDownColumns;
            }

            set
            {
                _minDropDownColumns = value;
            }
        }

        private int _maxDropDownColumns;
        public int MaxDropDownColumns
        {
            get
            {
                return _maxDropDownColumns;
            }

            set
            {
                _maxDropDownColumns = value;
            }
        }

        private int _minPreviewColumns;
        public int MinPreviewColumns
        {
            get
            {
                return _minPreviewColumns;
            }

            set
            {
                _minPreviewColumns = value;
            }
        }

        private int _maxPreviewColumns;
        public int MaxPreviewColumns
        {
            get
            {
                return _maxPreviewColumns;
            }

            set
            {
                _maxPreviewColumns = value;
            }
        }

        private int _preferredDropDownColumns;
        public int PreferredDropDownColumns
        {
            get
            {
                return _preferredDropDownColumns;
            }

            set
            {
                _preferredDropDownColumns = value;
            }
        }

        private bool _shouldDisplayGalleryPreview;
        public bool ShouldDisplayGalleryPreview
        {
            get
            {
                return _shouldDisplayGalleryPreview;
            }

            set
            {
                _shouldDisplayGalleryPreview = value;
            }
        }
    }
}
